@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,8 +21,7 @@
 
 package com.openexchange.authentication.imap.impl;
 
-import static com.openexchange.authentication.LoginExceptionCodes.INVALID_CREDENTIALS_MISSING_CONTEXT_MAPPING;
-import static com.openexchange.authentication.LoginExceptionCodes.INVALID_CREDENTIALS_MISSING_USER_MAPPING;
+import static com.openexchange.authentication.LoginExceptionCodes.INVALID_CREDENTIALS;
 import static com.openexchange.authentication.LoginExceptionCodes.UNKNOWN;
 import static com.openexchange.java.Autoboxing.I;
 import java.io.IOException;
@@ -43,7 +42,6 @@ import com.openexchange.authentication.AuthenticationService;
 import com.openexchange.authentication.LoginExceptionCodes;
 import com.openexchange.authentication.LoginInfo;
 import com.openexchange.config.ConfigurationService;
-import com.openexchange.config.ConfigurationServices;
 import com.openexchange.configuration.ConfigurationException;
 import com.openexchange.context.ContextService;
 import com.openexchange.exception.OXException;
@@ -57,6 +55,7 @@ import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.mailaccount.MailAccountStorageService;
 import com.openexchange.mailaccount.MailAccounts;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
+import com.openexchange.net.ssl.config.SSLConfigurationService;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.user.UserService;
 
@@ -100,7 +99,7 @@ public class IMAPAuthentication implements AuthenticationService {
 
     /**
      * Default constructor.
-     * 
+     *
      * @throws IOException if an I/O error is occurred
      */
     public IMAPAuthentication(ServiceLookup services) throws IOException {
@@ -117,7 +116,7 @@ public class IMAPAuthentication implements AuthenticationService {
         }
 
         // Initialize configuration properties
-        props = ConfigurationServices.loadPropertiesFrom(configService.getFileByName("imapauth.properties"));
+        props = configService.getFile("imapauth.properties");
     }
 
     @Override
@@ -211,8 +210,11 @@ public class IMAPAuthentication implements AuthenticationService {
                 contextInfo = contextInfo.toLowerCase();
             }
 
+            // The flag indicating whether IMAP server used for authenticating is the primary one
+            // Assume "true" if host is "localhost" by default.
+            boolean isPrimary = "localhost".equals(host);
+
             // Support for multiple IMAP servers
-            boolean isPrimary = false;
             boolean secure = false;
             if ("true".equalsIgnoreCase(props.getProperty(PropertyNames.USE_MULTIPLE.name))) {
                 // Resolve user/context
@@ -251,14 +253,16 @@ public class IMAPAuthentication implements AuthenticationService {
                 }
 
                 // Check if configured IMAP server is the primary IMAP server of the user
-                Optional<Context> optionalContext = optContext(contextInfo).opt();
-                if (optionalContext.isPresent()) {
-                    Context ctx = optionalContext.get();
-                    Optional<Integer> optionalUserId = optUser(userInfo, loginInfo, ctx).opt();
-                    if (optionalUserId.isPresent()) {
-                        Optional<MailAccount> optionalDefaultMailAccount = optMailAccount(optionalUserId.get().intValue(), ctx.getContextId()).opt();
-                        if (optionalDefaultMailAccount.isPresent() && MailAccounts.isEqualImapAccount(optionalDefaultMailAccount.get(), host, port.intValue())) {
-                            isPrimary = true;
+                if (!isPrimary) {
+                    Optional<Context> optionalContext = optContext(contextInfo).opt();
+                    if (optionalContext.isPresent()) {
+                        Context ctx = optionalContext.get();
+                        Optional<Integer> optionalUserId = optUser(userInfo, loginInfo, ctx).opt();
+                        if (optionalUserId.isPresent()) {
+                            Optional<MailAccount> optionalDefaultMailAccount = optMailAccount(optionalUserId.get().intValue(), ctx.getContextId()).opt();
+                            if (optionalDefaultMailAccount.isPresent() && MailAccounts.isEqualImapAccount(optionalDefaultMailAccount.get(), host, port.intValue())) {
+                                isPrimary = true;
+                            }
                         }
                     }
                 }
@@ -425,8 +429,15 @@ public class IMAPAuthentication implements AuthenticationService {
             imapprops.put("mail.imap.ssl.protocols", sslProtocols);
         }
 
-        sslProtocols = configuration.getProperty("com.openexchange.imap.ssl.protocols", "SSLv3 TLSv1").trim();
-        imapprops.put("mail.imap.ssl.protocols", sslProtocols);
+        sslProtocols = configuration.getProperty("com.openexchange.imap.ssl.protocols", "").trim();
+        if (Strings.isNotEmpty(sslProtocols)) {
+            imapprops.put("mail.imap.ssl.protocols", sslProtocols);
+        }
+
+        SSLConfigurationService sslConfigService = services.getOptionalService(SSLConfigurationService.class);
+        if (sslConfigService != null) {
+            imapprops.put("mail.imap.ssl.protocols", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedProtocols()));
+        }
     }
 
     private void applySslCipherSuites(Properties imapprops, ConfigurationService configuration) {
@@ -439,6 +450,11 @@ public class IMAPAuthentication implements AuthenticationService {
         if (Strings.isNotEmpty(cipherSuites)) {
             imapprops.put("mail.imap.ssl.ciphersuites", cipherSuites);
         }
+
+        SSLConfigurationService sslConfigService = services.getOptionalService(SSLConfigurationService.class);
+        if (sslConfigService != null) {
+            imapprops.put("mail.imap.ssl.ciphersuites", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedCipherSuites()));
+        }
     }
 
     private Result<Context> optContext(String contextInfo) {
@@ -446,7 +462,8 @@ public class IMAPAuthentication implements AuthenticationService {
             ContextService contextService = services.getService(ContextService.class);
             int ctxId = contextService.getContextId(contextInfo);
             if (ContextStorage.NOT_FOUND == ctxId) {
-                throw INVALID_CREDENTIALS_MISSING_CONTEXT_MAPPING.create(contextInfo);
+                LOG.debug("Missing context mapping for context \"{}\". Login failed.", contextInfo);
+                throw INVALID_CREDENTIALS.create(contextInfo);
             }
             return new Result<Context>(contextService.getContext(ctxId));
         } catch (OXException e) {
@@ -459,7 +476,8 @@ public class IMAPAuthentication implements AuthenticationService {
         try {
             return new Result<Integer>(I(userService.getUserId(userInfo, ctx)));
         } catch (OXException e) {
-            return new Result<Integer>(INVALID_CREDENTIALS_MISSING_USER_MAPPING.create(loginInfo.getUsername()));
+            LOG.debug("Missing user mapping for user \"{}\". Login failed.", loginInfo.getUsername());
+            return new Result<Integer>(INVALID_CREDENTIALS.create());
         }
     }
 
